@@ -2,6 +2,10 @@
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 import copy
+from sql import Column
+from sql.aggregate import Sum
+from sql.conditionals import Coalesce
+
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.pyson import Eval, PYSONEncoder
@@ -20,8 +24,8 @@ class Account(ModelSQL, ModelView):
     active = fields.Boolean('Active', select=True)
     company = fields.Many2One('company.company', 'Company')
     currency = fields.Many2One('currency.currency', 'Currency', required=True)
-    currency_digits = fields.Function(fields.Integer('Currency Digits',
-        on_change_with=['currency']), 'on_change_with_currency_digits')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+        'on_change_with_currency_digits')
     type = fields.Selection([
         ('root', 'Root'),
         ('view', 'View'),
@@ -107,6 +111,7 @@ class Account(ModelSQL, ModelView):
         super(Account, cls).validate(accounts)
         cls.check_recursion(accounts)
 
+    @fields.depends('currency')
     def on_change_with_currency_digits(self, name=None):
         if self.currency:
             return self.currency.digits
@@ -115,9 +120,18 @@ class Account(ModelSQL, ModelView):
     @classmethod
     def get_balance(cls, accounts, name):
         res = {}
-        Line = Pool().get('analytic_account.line')
-        Currency = Pool().get('currency.currency')
+        pool = Pool()
+        Line = pool.get('analytic_account.line')
+        MoveLine = pool.get('account.move.line')
+        Account = pool.get('account.account')
+        Company = pool.get('company.company')
+        Currency = pool.get('currency.currency')
         cursor = Transaction().cursor
+        table = cls.__table__()
+        line = Line.__table__()
+        move_line = MoveLine.__table__()
+        a_account = Account.__table__()
+        company = Company.__table__()
 
         ids = [a.id for a in accounts]
         childs = cls.search([('parent', 'child_of', ids)])
@@ -128,29 +142,29 @@ class Account(ModelSQL, ModelView):
         for account in all_accounts:
             id2account[account.id] = account
 
-        line_query = Line.query_get()
-        cursor.execute('SELECT a.id, '
-                'SUM((COALESCE(l.debit, 0) - COALESCE(l.credit, 0))), '
-                    'c.currency '
-            'FROM analytic_account_account a '
-                'LEFT JOIN analytic_account_line l '
-                'ON (a.id = l.account) '
-                'LEFT JOIN account_move_line ml '
-                'ON (ml.id = l.move_line) '
-                'LEFT JOIN account_account aa '
-                'ON (aa.id = ml.account) '
-                'LEFT JOIN company_company c '
-                'ON (c.id = aa.company) '
-            'WHERE a.type != \'view\' '
-                'AND a.id IN (' +
-                    ','.join(('%s',) * len(all_ids)) + ') '
-                'AND ' + line_query + ' '
-                'AND a.active '
-            'GROUP BY a.id, c.currency', all_ids)
+        line_query = Line.query_get(line)
+        cursor.execute(*table.join(line, 'LEFT',
+                condition=table.id == line.account
+                ).join(move_line, 'LEFT',
+                condition=move_line.id == line.move_line
+                ).join(a_account, 'LEFT',
+                condition=a_account.id == move_line.account
+                ).join(company, 'LEFT',
+                condition=company.id == a_account.company
+                ).select(table.id,
+                Sum(Coalesce(line.debit, 0) - Coalesce(line.credit, 0)),
+                company.currency,
+                where=(table.type != 'view')
+                & table.id.in_(all_ids)
+                & table.active & line_query,
+                group_by=(table.id, company.currency)))
         account_sum = {}
         id2currency = {}
         for account_id, sum, currency_id in cursor.fetchall():
             account_sum.setdefault(account_id, Decimal('0.0'))
+            # SQLite uses float for SUM
+            if not isinstance(sum, Decimal):
+                sum = Decimal(str(sum))
             if currency_id != id2account[account_id].currency.id:
                 currency = None
                 if currency_id in id2currency:
@@ -185,8 +199,16 @@ class Account(ModelSQL, ModelView):
         res = {}
         pool = Pool()
         Line = pool.get('analytic_account.line')
+        MoveLine = pool.get('account.move.line')
+        Account = pool.get('account.account')
+        Company = pool.get('company.company')
         Currency = pool.get('currency.currency')
         cursor = Transaction().cursor
+        table = cls.__table__()
+        line = Line.__table__()
+        move_line = MoveLine.__table__()
+        a_account = Account.__table__()
+        company = Company.__table__()
 
         if name not in ('credit', 'debit'):
             raise Exception('Bad argument')
@@ -197,28 +219,28 @@ class Account(ModelSQL, ModelView):
             res[account.id] = Decimal('0.0')
             id2account[account.id] = account
 
-        line_query = Line.query_get()
-        cursor.execute('SELECT a.id, '
-                'SUM(COALESCE(l.' + name + ', 0)), '
-                'c.currency '
-            'FROM analytic_account_account a '
-                'LEFT JOIN analytic_account_line l '
-                'ON (a.id = l.account) '
-                'LEFT JOIN account_move_line ml '
-                'ON (ml.id = l.move_line) '
-                'LEFT JOIN account_account aa '
-                'ON (aa.id = ml.account) '
-                'LEFT JOIN company_company c '
-                'ON (c.id = aa.company) '
-            'WHERE a.type != \'view\' '
-                'AND a.id IN (' +
-                    ','.join(('%s',) * len(ids)) + ') '
-                'AND ' + line_query + ' '
-                'AND a.active '
-            'GROUP BY a.id, c.currency', ids)
+        line_query = Line.query_get(line)
+        cursor.execute(*table.join(line, 'LEFT',
+                condition=table.id == line.account
+                ).join(move_line, 'LEFT',
+                condition=move_line.id == line.move_line
+                ).join(a_account, 'LEFT',
+                condition=a_account.id == move_line.account
+                ).join(company, 'LEFT',
+                condition=company.id == a_account.company
+                ).select(table.id,
+                Sum(Coalesce(Column(line, name), 0)),
+                company.currency,
+                where=(table.type != 'view')
+                & table.id.in_(ids)
+                & table.active & line_query,
+                group_by=(table.id, company.currency)))
 
         id2currency = {}
         for account_id, sum, currency_id in cursor.fetchall():
+            # SQLite uses float for SUM
+            if not isinstance(sum, Decimal):
+                sum = Decimal(str(sum))
             if currency_id != id2account[account_id].currency.id:
                 currency = None
                 if currency_id in id2currency:
@@ -240,11 +262,11 @@ class Account(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        accounts = cls.search([('code',) + clause[1:]], limit=1)
+        accounts = cls.search([('code',) + tuple(clause[1:])], limit=1)
         if accounts:
-            return [('code',) + clause[1:]]
+            return [('code',) + tuple(clause[1:])]
         else:
-            return [(cls._rec_name,) + clause[1:]]
+            return [(cls._rec_name,) + tuple(clause[1:])]
 
     @classmethod
     def convert_view(cls, tree):
@@ -272,10 +294,15 @@ class Account(ModelSQL, ModelView):
         parent.remove(element_accounts)
 
     @classmethod
-    def analytic_accounts_fields_get(cls, field, fields_names=None):
+    def analytic_accounts_fields_get(cls, field, fields_names=None,
+            states=None, required_states=None):
         res = {}
         if fields_names is None:
             fields_names = []
+        if states is None:
+            states = {}
+
+        encoder = PYSONEncoder()
 
         root_accounts = cls.search([
                 ('parent', '=', None),
@@ -284,7 +311,13 @@ class Account(ModelSQL, ModelView):
             name = 'analytic_account_' + str(account.id)
             if name in fields_names or not fields_names:
                 res[name] = field.copy()
-                res[name]['required'] = account.mandatory
+                field_states = states.copy()
+                if account.mandatory:
+                    if required_states:
+                        field_states['required'] = required_states
+                    else:
+                        field_states['required'] = True
+                res[name]['states'] = encoder.encode(field_states)
                 res[name]['string'] = account.name
                 res[name]['relation'] = cls.__name__
                 res[name]['domain'] = PYSONEncoder().encode([
